@@ -3,22 +3,22 @@ use editor::Editor;
 use extension::ExtensionStore;
 use futures::StreamExt;
 use gpui::{
-    actions, svg, AppContext, CursorStyle, EventEmitter, InteractiveElement as _, Model,
-    ParentElement as _, Render, SharedString, StatefulInteractiveElement, Styled, View,
-    ViewContext, VisualContext as _,
+    actions, anchored, deferred, percentage, Animation, AnimationExt as _, AppContext, CursorStyle,
+    DismissEvent, EventEmitter, InteractiveElement as _, Model, ParentElement as _, Render,
+    SharedString, StatefulInteractiveElement, Styled, Transformation, View, ViewContext,
+    VisualContext as _,
+};
+use language::{
+    LanguageRegistry, LanguageServerBinaryStatus, LanguageServerId, LanguageServerName,
 };
-use language::{LanguageRegistry, LanguageServerBinaryStatus, LanguageServerName};
 use project::{LanguageServerProgress, Project};
 use smallvec::SmallVec;
-use std::{cmp::Reverse, fmt::Write, sync::Arc};
-use ui::prelude::*;
+use std::{cmp::Reverse, fmt::Write, sync::Arc, time::Duration};
+use ui::{prelude::*, ContextMenu};
 use workspace::{item::ItemHandle, StatusItemView, Workspace};
 
 actions!(activity_indicator, [ShowErrorMessage]);
 
-const DOWNLOAD_ICON: &str = "icons/download.svg";
-const WARNING_ICON: &str = "icons/warning.svg";
-
 pub enum Event {
     ShowError { lsp_name: Arc<str>, error: String },
 }
@@ -27,6 +27,7 @@ pub struct ActivityIndicator {
     statuses: Vec<LspStatus>,
     project: Model<Project>,
     auto_updater: Option<Model<AutoUpdater>>,
+    context_menu: Option<View<ContextMenu>>,
 }
 
 struct LspStatus {
@@ -35,14 +36,14 @@ struct LspStatus {
 }
 
 struct PendingWork<'a> {
-    language_server_name: &'a str,
+    language_server_id: LanguageServerId,
     progress_token: &'a str,
     progress: &'a LanguageServerProgress,
 }
 
 #[derive(Default)]
 struct Content {
-    icon: Option<&'static str>,
+    icon: Option<gpui::AnyElement>,
     message: String,
     on_click: Option<Arc<dyn Fn(&mut ActivityIndicator, &mut ViewContext<ActivityIndicator>)>>,
 }
@@ -78,6 +79,7 @@ impl ActivityIndicator {
                 statuses: Default::default(),
                 project: project.clone(),
                 auto_updater,
+                context_menu: None,
             }
         });
 
@@ -151,7 +153,7 @@ impl ActivityIndicator {
             .read(cx)
             .language_server_statuses()
             .rev()
-            .filter_map(|status| {
+            .filter_map(|(server_id, status)| {
                 if status.pending_work.is_empty() {
                     None
                 } else {
@@ -159,7 +161,7 @@ impl ActivityIndicator {
                         .pending_work
                         .iter()
                         .map(|(token, progress)| PendingWork {
-                            language_server_name: status.name.as_str(),
+                            language_server_id: server_id,
                             progress_token: token.as_str(),
                             progress,
                         })
@@ -175,33 +177,44 @@ impl ActivityIndicator {
         // Show any language server has pending activity.
         let mut pending_work = self.pending_language_server_work(cx);
         if let Some(PendingWork {
-            language_server_name,
             progress_token,
             progress,
+            ..
         }) = pending_work.next()
         {
-            let mut message = language_server_name.to_string();
-
-            message.push_str(": ");
-            if let Some(progress_message) = progress.message.as_ref() {
-                message.push_str(progress_message);
-            } else {
-                message.push_str(progress_token);
-            }
+            let mut message = progress
+                .title
+                .as_deref()
+                .unwrap_or(progress_token)
+                .to_string();
 
             if let Some(percentage) = progress.percentage {
                 write!(&mut message, " ({}%)", percentage).unwrap();
             }
 
+            if let Some(progress_message) = progress.message.as_ref() {
+                message.push_str(": ");
+                message.push_str(progress_message);
+            }
+
             let additional_work_count = pending_work.count();
             if additional_work_count > 0 {
                 write!(&mut message, " + {} more", additional_work_count).unwrap();
             }
 
             return Content {
-                icon: None,
+                icon: Some(
+                    Icon::new(IconName::ArrowCircle)
+                        .size(IconSize::Small)
+                        .with_animation(
+                            "arrow-circle",
+                            Animation::new(Duration::from_secs(2)).repeat(),
+                            |icon, delta| icon.transform(Transformation::rotate(percentage(delta))),
+                        )
+                        .into_any_element(),
+                ),
                 message,
-                on_click: None,
+                on_click: Some(Arc::new(Self::toggle_language_server_work_context_menu)),
             };
         }
 
@@ -222,7 +235,11 @@ impl ActivityIndicator {
 
         if !downloading.is_empty() {
             return Content {
-                icon: Some(DOWNLOAD_ICON),
+                icon: Some(
+                    Icon::new(IconName::Download)
+                        .size(IconSize::Small)
+                        .into_any_element(),
+                ),
                 message: format!("Downloading {}...", downloading.join(", "),),
                 on_click: None,
             };
@@ -230,7 +247,11 @@ impl ActivityIndicator {
 
         if !checking_for_update.is_empty() {
             return Content {
-                icon: Some(DOWNLOAD_ICON),
+                icon: Some(
+                    Icon::new(IconName::Download)
+                        .size(IconSize::Small)
+                        .into_any_element(),
+                ),
                 message: format!(
                     "Checking for updates to {}...",
                     checking_for_update.join(", "),
@@ -241,7 +262,11 @@ impl ActivityIndicator {
 
         if !failed.is_empty() {
             return Content {
-                icon: Some(WARNING_ICON),
+                icon: Some(
+                    Icon::new(IconName::ExclamationTriangle)
+                        .size(IconSize::Small)
+                        .into_any_element(),
+                ),
                 message: format!(
                     "Failed to download {}. Click to show error.",
                     failed.join(", "),
@@ -255,7 +280,11 @@ impl ActivityIndicator {
         // Show any formatting failure
         if let Some(failure) = self.project.read(cx).last_formatting_failure() {
             return Content {
-                icon: Some(WARNING_ICON),
+                icon: Some(
+                    Icon::new(IconName::ExclamationTriangle)
+                        .size(IconSize::Small)
+                        .into_any_element(),
+                ),
                 message: format!("Formatting failed: {}. Click to see logs.", failure),
                 on_click: Some(Arc::new(|_, cx| {
                     cx.dispatch_action(Box::new(workspace::OpenLog));
@@ -267,17 +296,29 @@ impl ActivityIndicator {
         if let Some(updater) = &self.auto_updater {
             return match &updater.read(cx).status() {
                 AutoUpdateStatus::Checking => Content {
-                    icon: Some(DOWNLOAD_ICON),
+                    icon: Some(
+                        Icon::new(IconName::Download)
+                            .size(IconSize::Small)
+                            .into_any_element(),
+                    ),
                     message: "Checking for Zed updates…".to_string(),
                     on_click: None,
                 },
                 AutoUpdateStatus::Downloading => Content {
-                    icon: Some(DOWNLOAD_ICON),
+                    icon: Some(
+                        Icon::new(IconName::Download)
+                            .size(IconSize::Small)
+                            .into_any_element(),
+                    ),
                     message: "Downloading Zed update…".to_string(),
                     on_click: None,
                 },
                 AutoUpdateStatus::Installing => Content {
-                    icon: Some(DOWNLOAD_ICON),
+                    icon: Some(
+                        Icon::new(IconName::Download)
+                            .size(IconSize::Small)
+                            .into_any_element(),
+                    ),
                     message: "Installing Zed update…".to_string(),
                     on_click: None,
                 },
@@ -292,7 +333,11 @@ impl ActivityIndicator {
                     })),
                 },
                 AutoUpdateStatus::Errored => Content {
-                    icon: Some(WARNING_ICON),
+                    icon: Some(
+                        Icon::new(IconName::ExclamationTriangle)
+                            .size(IconSize::Small)
+                            .into_any_element(),
+                    ),
                     message: "Auto update failed".to_string(),
                     on_click: Some(Arc::new(|this, cx| {
                         this.dismiss_error_message(&Default::default(), cx)
@@ -307,7 +352,11 @@ impl ActivityIndicator {
         {
             if let Some(extension_id) = extension_store.outstanding_operations().keys().next() {
                 return Content {
-                    icon: Some(DOWNLOAD_ICON),
+                    icon: Some(
+                        Icon::new(IconName::Download)
+                            .size(IconSize::Small)
+                            .into_any_element(),
+                    ),
                     message: format!("Updating {extension_id} extension…"),
                     on_click: None,
                 };
@@ -316,6 +365,75 @@ impl ActivityIndicator {
 
         Default::default()
     }
+
+    fn toggle_language_server_work_context_menu(&mut self, cx: &mut ViewContext<Self>) {
+        if self.context_menu.take().is_some() {
+            return;
+        }
+
+        self.build_lsp_work_context_menu(cx);
+        cx.notify();
+    }
+
+    fn build_lsp_work_context_menu(&mut self, cx: &mut ViewContext<Self>) {
+        let mut has_work = false;
+        let this = cx.view().downgrade();
+        let context_menu = ContextMenu::build(cx, |mut menu, cx| {
+            for work in self.pending_language_server_work(cx) {
+                has_work = true;
+
+                let this = this.clone();
+                let title = SharedString::from(
+                    work.progress
+                        .title
+                        .as_deref()
+                        .unwrap_or(work.progress_token)
+                        .to_string(),
+                );
+                if work.progress.is_cancellable {
+                    let language_server_id = work.language_server_id;
+                    let token = work.progress_token.to_string();
+                    menu = menu.custom_entry(
+                        move |_| {
+                            h_flex()
+                                .w_full()
+                                .justify_between()
+                                .child(Label::new(title.clone()))
+                                .child(Icon::new(IconName::XCircle))
+                                .into_any_element()
+                        },
+                        move |cx| {
+                            this.update(cx, |this, cx| {
+                                this.project.update(cx, |project, cx| {
+                                    project.cancel_language_server_work(
+                                        language_server_id,
+                                        Some(token.clone()),
+                                        cx,
+                                    );
+                                });
+                                this.context_menu.take();
+                            })
+                            .ok();
+                        },
+                    );
+                } else {
+                    menu = menu.label(title.clone());
+                }
+            }
+            menu
+        });
+
+        if has_work {
+            cx.subscribe(&context_menu, |this, _, _: &DismissEvent, cx| {
+                this.context_menu.take();
+                cx.notify();
+            })
+            .detach();
+            cx.focus_view(&context_menu);
+            self.context_menu = Some(context_menu);
+            cx.notify();
+        }
+    }
 }
 
 impl EventEmitter<Event> for ActivityIndicator {}
@@ -338,8 +456,17 @@ impl Render for ActivityIndicator {
         }
 
         result
-            .children(content.icon.map(|icon| svg().path(icon)))
+            .gap_2()
+            .children(content.icon)
             .child(Label::new(SharedString::from(content.message)).size(LabelSize::Small))
+            .children(self.context_menu.as_ref().map(|menu| {
+                deferred(
+                    anchored()
+                        .anchor(gpui::AnchorCorner::BottomLeft)
+                        .child(menu.clone()),
+                )
+                .with_priority(1)
+            }))
     }
 }
 
