@@ -308,13 +308,12 @@ pub async fn post_panic(
         .map_err(|_| Error::Http(StatusCode::BAD_REQUEST, "invalid json".into()))?;
     let panic = report.panic;
 
-    // better OS reporting for linux (because linux is hard):
-    // - Remove os_version/app_version/os_name from the gpui platform trait
-    // - Move platform processing data into client/telemetry
-    // - Duplicate some small code in macOS platform for a version check
-    // - Add GPUI API for reporting the selected platform integration
-    //  - macos-blade, macos-metal, linux-X11, linux-headless
-    // if cfg(macos( { "Macos" } else { "Linux-{cx.compositor_name()"} ))
+    if panic.os_name == "Linux" && panic.os_version == Some("1.0.0".to_string()) {
+        return Err(Error::Http(
+            StatusCode::BAD_REQUEST,
+            "invalid os version".into(),
+        ))?;
+    }
 
     tracing::error!(
         service = "client",
@@ -402,12 +401,7 @@ pub async fn post_events(
         ))?;
     };
 
-    if checksum != expected {
-        return Err(Error::Http(
-            StatusCode::BAD_REQUEST,
-            "invalid checksum".into(),
-        ))?;
-    }
+    let checksum_matched = checksum == expected;
 
     let request_body: telemetry_events::EventRequestBody =
         serde_json::from_slice(&body).map_err(|err| {
@@ -432,6 +426,7 @@ pub async fn post_events(
                 &request_body,
                 first_event_at,
                 country_code.clone(),
+                checksum_matched,
             )),
             // Needed for clients sending old copilot_event types
             Event::Copilot(_) => {}
@@ -444,6 +439,7 @@ pub async fn post_events(
                         &request_body,
                         first_event_at,
                         country_code.clone(),
+                        checksum_matched,
                     ))
             }
             Event::Call(event) => to_upload.call_events.push(CallEventRow::from_event(
@@ -451,6 +447,7 @@ pub async fn post_events(
                 &wrapper,
                 &request_body,
                 first_event_at,
+                checksum_matched,
             )),
             Event::Assistant(event) => {
                 to_upload
@@ -460,6 +457,7 @@ pub async fn post_events(
                         &wrapper,
                         &request_body,
                         first_event_at,
+                        checksum_matched,
                     ))
             }
             Event::Cpu(event) => to_upload.cpu_events.push(CpuEventRow::from_event(
@@ -467,36 +465,42 @@ pub async fn post_events(
                 &wrapper,
                 &request_body,
                 first_event_at,
+                checksum_matched,
             )),
             Event::Memory(event) => to_upload.memory_events.push(MemoryEventRow::from_event(
                 event.clone(),
                 &wrapper,
                 &request_body,
                 first_event_at,
+                checksum_matched,
             )),
             Event::App(event) => to_upload.app_events.push(AppEventRow::from_event(
                 event.clone(),
                 &wrapper,
                 &request_body,
                 first_event_at,
+                checksum_matched,
             )),
             Event::Setting(event) => to_upload.setting_events.push(SettingEventRow::from_event(
                 event.clone(),
                 &wrapper,
                 &request_body,
                 first_event_at,
+                checksum_matched,
             )),
             Event::Edit(event) => to_upload.edit_events.push(EditEventRow::from_event(
                 event.clone(),
                 &wrapper,
                 &request_body,
                 first_event_at,
+                checksum_matched,
             )),
             Event::Action(event) => to_upload.action_events.push(ActionEventRow::from_event(
                 event.clone(),
                 &wrapper,
                 &request_body,
                 first_event_at,
+                checksum_matched,
             )),
             Event::Extension(event) => {
                 let metadata = app
@@ -511,6 +515,7 @@ pub async fn post_events(
                         &request_body,
                         metadata,
                         first_event_at,
+                        checksum_matched,
                     ))
             }
         }
@@ -658,29 +663,31 @@ where
 
 #[derive(Serialize, Debug, clickhouse::Row)]
 pub struct EditorEventRow {
-    pub installation_id: String,
-    pub operation: String,
-    pub app_version: String,
-    pub file_extension: String,
-    pub os_name: String,
-    pub os_version: String,
-    pub release_channel: String,
-    pub signed_in: bool,
-    pub vim_mode: bool,
+    installation_id: String,
+    metrics_id: String,
+    operation: String,
+    app_version: String,
+    file_extension: String,
+    os_name: String,
+    os_version: String,
+    release_channel: String,
+    signed_in: bool,
+    vim_mode: bool,
     #[serde(serialize_with = "serialize_country_code")]
-    pub country_code: String,
-    pub region_code: String,
-    pub city: String,
-    pub time: i64,
-    pub copilot_enabled: bool,
-    pub copilot_enabled_for_language: bool,
-    pub historical_event: bool,
-    pub architecture: String,
-    pub is_staff: Option<bool>,
-    pub session_id: Option<String>,
-    pub major: Option<i32>,
-    pub minor: Option<i32>,
-    pub patch: Option<i32>,
+    country_code: String,
+    region_code: String,
+    city: String,
+    time: i64,
+    copilot_enabled: bool,
+    copilot_enabled_for_language: bool,
+    historical_event: bool,
+    architecture: String,
+    is_staff: Option<bool>,
+    session_id: Option<String>,
+    major: Option<i32>,
+    minor: Option<i32>,
+    patch: Option<i32>,
+    checksum_matched: bool,
 }
 
 impl EditorEventRow {
@@ -690,6 +697,7 @@ impl EditorEventRow {
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
         country_code: Option<String>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -700,11 +708,13 @@ impl EditorEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
             os_name: body.os_name.clone(),
             os_version: body.os_version.clone().unwrap_or_default(),
             architecture: body.architecture.clone(),
             installation_id: body.installation_id.clone().unwrap_or_default(),
+            metrics_id: body.metrics_id.clone().unwrap_or_default(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
             time: time.timestamp_millis(),
@@ -724,25 +734,26 @@ impl EditorEventRow {
 
 #[derive(Serialize, Debug, clickhouse::Row)]
 pub struct InlineCompletionEventRow {
-    pub installation_id: String,
-    pub provider: String,
-    pub suggestion_accepted: bool,
-    pub app_version: String,
-    pub file_extension: String,
-    pub os_name: String,
-    pub os_version: String,
-    pub release_channel: String,
-    pub signed_in: bool,
+    installation_id: String,
+    provider: String,
+    suggestion_accepted: bool,
+    app_version: String,
+    file_extension: String,
+    os_name: String,
+    os_version: String,
+    release_channel: String,
+    signed_in: bool,
     #[serde(serialize_with = "serialize_country_code")]
-    pub country_code: String,
-    pub region_code: String,
-    pub city: String,
-    pub time: i64,
-    pub is_staff: Option<bool>,
-    pub session_id: Option<String>,
-    pub major: Option<i32>,
-    pub minor: Option<i32>,
-    pub patch: Option<i32>,
+    country_code: String,
+    region_code: String,
+    city: String,
+    time: i64,
+    is_staff: Option<bool>,
+    session_id: Option<String>,
+    major: Option<i32>,
+    minor: Option<i32>,
+    patch: Option<i32>,
+    checksum_matched: bool,
 }
 
 impl InlineCompletionEventRow {
@@ -752,6 +763,7 @@ impl InlineCompletionEventRow {
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
         country_code: Option<String>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -762,6 +774,7 @@ impl InlineCompletionEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
             os_name: body.os_name.clone(),
             os_version: body.os_version.clone().unwrap_or_default(),
@@ -788,6 +801,9 @@ pub struct CallEventRow {
     minor: Option<i32>,
     patch: Option<i32>,
     release_channel: String,
+    os_name: String,
+    os_version: String,
+    checksum_matched: bool,
 
     // ClientEventBase
     installation_id: String,
@@ -807,6 +823,7 @@ impl CallEventRow {
         wrapper: &EventWrapper,
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -817,7 +834,10 @@ impl CallEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
+            os_name: body.os_name.clone(),
+            os_version: body.os_version.clone().unwrap_or_default(),
             installation_id: body.installation_id.clone().unwrap_or_default(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
@@ -836,7 +856,10 @@ pub struct AssistantEventRow {
     major: Option<i32>,
     minor: Option<i32>,
     patch: Option<i32>,
+    checksum_matched: bool,
     release_channel: String,
+    os_name: String,
+    os_version: String,
 
     // ClientEventBase
     installation_id: Option<String>,
@@ -858,6 +881,7 @@ impl AssistantEventRow {
         wrapper: &EventWrapper,
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -868,7 +892,10 @@ impl AssistantEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
+            os_name: body.os_name.clone(),
+            os_version: body.os_version.clone().unwrap_or_default(),
             installation_id: body.installation_id.clone(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
@@ -886,18 +913,21 @@ impl AssistantEventRow {
 
 #[derive(Debug, clickhouse::Row, Serialize)]
 pub struct CpuEventRow {
-    pub installation_id: Option<String>,
-    pub is_staff: Option<bool>,
-    pub usage_as_percentage: f32,
-    pub core_count: u32,
-    pub app_version: String,
-    pub release_channel: String,
-    pub time: i64,
-    pub session_id: Option<String>,
+    installation_id: Option<String>,
+    is_staff: Option<bool>,
+    usage_as_percentage: f32,
+    core_count: u32,
+    app_version: String,
+    release_channel: String,
+    os_name: String,
+    os_version: String,
+    time: i64,
+    session_id: Option<String>,
     // pub normalized_cpu_usage: f64, MATERIALIZED
-    pub major: Option<i32>,
-    pub minor: Option<i32>,
-    pub patch: Option<i32>,
+    major: Option<i32>,
+    minor: Option<i32>,
+    patch: Option<i32>,
+    checksum_matched: bool,
 }
 
 impl CpuEventRow {
@@ -906,6 +936,7 @@ impl CpuEventRow {
         wrapper: &EventWrapper,
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -916,7 +947,10 @@ impl CpuEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
+            os_name: body.os_name.clone(),
+            os_version: body.os_version.clone().unwrap_or_default(),
             installation_id: body.installation_id.clone(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
@@ -934,7 +968,10 @@ pub struct MemoryEventRow {
     major: Option<i32>,
     minor: Option<i32>,
     patch: Option<i32>,
+    checksum_matched: bool,
     release_channel: String,
+    os_name: String,
+    os_version: String,
 
     // ClientEventBase
     installation_id: Option<String>,
@@ -953,6 +990,7 @@ impl MemoryEventRow {
         wrapper: &EventWrapper,
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -963,7 +1001,10 @@ impl MemoryEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
+            os_name: body.os_name.clone(),
+            os_version: body.os_version.clone().unwrap_or_default(),
             installation_id: body.installation_id.clone(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
@@ -981,7 +1022,10 @@ pub struct AppEventRow {
     major: Option<i32>,
     minor: Option<i32>,
     patch: Option<i32>,
+    checksum_matched: bool,
     release_channel: String,
+    os_name: String,
+    os_version: String,
 
     // ClientEventBase
     installation_id: Option<String>,
@@ -999,6 +1043,7 @@ impl AppEventRow {
         wrapper: &EventWrapper,
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -1009,7 +1054,10 @@ impl AppEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
+            os_name: body.os_name.clone(),
+            os_version: body.os_version.clone().unwrap_or_default(),
             installation_id: body.installation_id.clone(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
@@ -1026,7 +1074,10 @@ pub struct SettingEventRow {
     major: Option<i32>,
     minor: Option<i32>,
     patch: Option<i32>,
+    checksum_matched: bool,
     release_channel: String,
+    os_name: String,
+    os_version: String,
 
     // ClientEventBase
     installation_id: Option<String>,
@@ -1044,6 +1095,7 @@ impl SettingEventRow {
         wrapper: &EventWrapper,
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -1053,8 +1105,11 @@ impl SettingEventRow {
             app_version: body.app_version.clone(),
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
+            checksum_matched,
             patch: semver.map(|v| v.patch() as i32),
             release_channel: body.release_channel.clone().unwrap_or_default(),
+            os_name: body.os_name.clone(),
+            os_version: body.os_version.clone().unwrap_or_default(),
             installation_id: body.installation_id.clone(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
@@ -1072,7 +1127,10 @@ pub struct ExtensionEventRow {
     major: Option<i32>,
     minor: Option<i32>,
     patch: Option<i32>,
+    checksum_matched: bool,
     release_channel: String,
+    os_name: String,
+    os_version: String,
 
     // ClientEventBase
     installation_id: Option<String>,
@@ -1095,6 +1153,7 @@ impl ExtensionEventRow {
         body: &EventRequestBody,
         extension_metadata: Option<ExtensionMetadata>,
         first_event_at: chrono::DateTime<chrono::Utc>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -1105,7 +1164,10 @@ impl ExtensionEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
+            os_name: body.os_name.clone(),
+            os_version: body.os_version.clone().unwrap_or_default(),
             installation_id: body.installation_id.clone(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
@@ -1134,7 +1196,10 @@ pub struct EditEventRow {
     major: Option<i32>,
     minor: Option<i32>,
     patch: Option<i32>,
+    checksum_matched: bool,
     release_channel: String,
+    os_name: String,
+    os_version: String,
 
     // ClientEventBase
     installation_id: Option<String>,
@@ -1156,6 +1221,7 @@ impl EditEventRow {
         wrapper: &EventWrapper,
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -1169,7 +1235,10 @@ impl EditEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
+            os_name: body.os_name.clone(),
+            os_version: body.os_version.clone().unwrap_or_default(),
             installation_id: body.installation_id.clone(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
@@ -1188,7 +1257,10 @@ pub struct ActionEventRow {
     major: Option<i32>,
     minor: Option<i32>,
     patch: Option<i32>,
+    checksum_matched: bool,
     release_channel: String,
+    os_name: String,
+    os_version: String,
 
     // ClientEventBase
     installation_id: Option<String>,
@@ -1208,6 +1280,7 @@ impl ActionEventRow {
         wrapper: &EventWrapper,
         body: &EventRequestBody,
         first_event_at: chrono::DateTime<chrono::Utc>,
+        checksum_matched: bool,
     ) -> Self {
         let semver = body.semver();
         let time =
@@ -1218,7 +1291,10 @@ impl ActionEventRow {
             major: semver.map(|v| v.major() as i32),
             minor: semver.map(|v| v.minor() as i32),
             patch: semver.map(|v| v.patch() as i32),
+            checksum_matched,
             release_channel: body.release_channel.clone().unwrap_or_default(),
+            os_name: body.os_name.clone(),
+            os_version: body.os_version.clone().unwrap_or_default(),
             installation_id: body.installation_id.clone(),
             session_id: body.session_id.clone(),
             is_staff: body.is_staff,
