@@ -3,9 +3,8 @@ use crate::notifications::collab_notification::CollabNotification;
 use call::{ActiveCall, IncomingCall};
 use futures::StreamExt;
 use gpui::{prelude::*, AppContext, WindowHandle};
-use settings::Settings;
+
 use std::sync::{Arc, Weak};
-use theme::ThemeSettings;
 use ui::{prelude::*, Button, Label};
 use util::ResultExt;
 use workspace::AppState;
@@ -113,13 +112,7 @@ impl IncomingCallNotification {
 
 impl Render for IncomingCallNotification {
     fn render(&mut self, cx: &mut ViewContext<Self>) -> impl IntoElement {
-        // TODO: Is there a better place for us to initialize the font?
-        let (ui_font, ui_font_size) = {
-            let theme_settings = ThemeSettings::get_global(cx);
-            (theme_settings.ui_font.clone(), theme_settings.ui_font_size)
-        };
-
-        cx.set_rem_size(ui_font_size);
+        let ui_font = theme::setup_ui_font(cx);
 
         div().size_full().font(ui_font).child(
             CollabNotification::new(
