@@ -2288,6 +2288,18 @@ impl Pixels {
         Self(self.0.abs())
     }
 
+    /// Returns the sign of the `Pixels` value.
+    ///
+    /// # Returns
+    ///
+    /// Returns:
+    /// * `1.0` if the value is positive
+    /// * `-1.0` if the value is negative
+    /// * `0.0` if the value is zero
+    pub fn signum(&self) -> f32 {
+        self.0.signum()
+    }
+
     /// Returns the f64 value of `Pixels`.
     ///
     /// # Returns
