@@ -1,7 +1,8 @@
-use crate::{ErrorCode, ErrorCodeExt, ErrorExt, RpcError};
-
 use super::{
-    proto::{self, AnyTypedEnvelope, EnvelopedMessage, MessageStream, PeerId, RequestMessage},
+    proto::{
+        self, AnyTypedEnvelope, EnvelopedMessage, MessageStream, PeerId, Receipt, RequestMessage,
+        TypedEnvelope,
+    },
     Connection,
 };
 use anyhow::{anyhow, Context, Result};
@@ -12,11 +13,11 @@ use futures::{
     FutureExt, SinkExt, Stream, StreamExt, TryFutureExt,
 };
 use parking_lot::{Mutex, RwLock};
+use proto::{ErrorCode, ErrorCodeExt, ErrorExt, RpcError};
 use serde::{ser::SerializeStruct, Serialize};
 use std::{
     fmt, future,
     future::Future,
-    marker::PhantomData,
     sync::atomic::Ordering::SeqCst,
     sync::{
         atomic::{self, AtomicU32},
@@ -57,46 +58,6 @@ impl fmt::Display for ConnectionId {
     }
 }
 
-pub struct Receipt<T> {
-    pub sender_id: ConnectionId,
-    pub message_id: u32,
-    payload_type: PhantomData<T>,
-}
-
-impl<T> Clone for Receipt<T> {
-    fn clone(&self) -> Self {
-        *self
-    }
-}
-
-impl<T> Copy for Receipt<T> {}
-
-#[derive(Clone, Debug)]
-pub struct TypedEnvelope<T> {
-    pub sender_id: ConnectionId,
-    pub original_sender_id: Option<PeerId>,
-    pub message_id: u32,
-    pub payload: T,
-    pub received_at: Instant,
-}
-
-impl<T> TypedEnvelope<T> {
-    pub fn original_sender_id(&self) -> Result<PeerId> {
-        self.original_sender_id
-            .ok_or_else(|| anyhow!("missing original_sender_id"))
-    }
-}
-
-impl<T: RequestMessage> TypedEnvelope<T> {
-    pub fn receipt(&self) -> Receipt<T> {
-        Receipt {
-            sender_id: self.sender_id,
-            message_id: self.message_id,
-            payload_type: PhantomData,
-        }
-    }
-}
-
 pub struct Peer {
     epoch: AtomicU32,
     pub connections: RwLock<HashMap<ConnectionId, ConnectionState>>,
@@ -376,9 +337,12 @@ impl Peer {
                             "incoming stream response: requester resumed"
                         );
                     } else {
-                        let message_type =
-                            proto::build_typed_envelope(connection_id, received_at, incoming)
-                                .map(|p| p.payload_type_name());
+                        let message_type = proto::build_typed_envelope(
+                            connection_id.into(),
+                            received_at,
+                            incoming,
+                        )
+                        .map(|p| p.payload_type_name());
                         tracing::warn!(
                             %connection_id,
                             message_id,
@@ -391,16 +355,15 @@ impl Peer {
                     None
                 } else {
                     tracing::trace!(%connection_id, message_id, "incoming message: received");
-                    proto::build_typed_envelope(connection_id, received_at, incoming).or_else(
-                        || {
+                    proto::build_typed_envelope(connection_id.into(), received_at, incoming)
+                        .or_else(|| {
                             tracing::error!(
                                 %connection_id,
                                 message_id,
                                 "unable to construct a typed envelope"
                             );
                             None
-                        },
-                    )
+                        })
                 }
             }
         });
@@ -435,6 +398,7 @@ impl Peer {
         self.connections.write().clear();
     }
 
+    /// Make a request and wait for a response.
     pub fn request<T: RequestMessage>(
         &self,
         receiver_id: ConnectionId,
@@ -462,28 +426,50 @@ impl Peer {
             .map_ok(|envelope| envelope.payload)
     }
 
-    pub fn request_internal<T: RequestMessage>(
+    fn request_internal<T: RequestMessage>(
         &self,
         original_sender_id: Option<ConnectionId>,
         receiver_id: ConnectionId,
         request: T,
     ) -> impl Future<Output = Result<TypedEnvelope<T::Response>>> {
+        let envelope = request.into_envelope(0, None, original_sender_id.map(Into::into));
+        let response = self.request_dynamic(receiver_id, envelope, T::NAME);
+        async move {
+            let (response, received_at) = response.await?;
+            Ok(TypedEnvelope {
+                message_id: response.id,
+                sender_id: receiver_id.into(),
+                original_sender_id: response.original_sender_id,
+                payload: T::Response::from_envelope(response)
+                    .ok_or_else(|| anyhow!("received response of the wrong type"))?,
+                received_at,
+            })
+        }
+    }
+
+    /// Make a request and wait for a response.
+    ///
+    /// The caller must make sure to deserialize the response into the request's
+    /// response type. This interface is only useful in trait objects, where
+    /// generics can't be used. If you have a concrete type, use `request`.
+    pub fn request_dynamic(
+        &self,
+        receiver_id: ConnectionId,
+        mut envelope: proto::Envelope,
+        type_name: &'static str,
+    ) -> impl Future<Output = Result<(proto::Envelope, Instant)>> {
         let (tx, rx) = oneshot::channel();
         let send = self.connection_state(receiver_id).and_then(|connection| {
-            let message_id = connection.next_message_id.fetch_add(1, SeqCst);
+            envelope.id = connection.next_message_id.fetch_add(1, SeqCst);
             connection
                 .response_channels
                 .lock()
                 .as_mut()
                 .ok_or_else(|| anyhow!("connection was closed"))?
-                .insert(message_id, tx);
+                .insert(envelope.id, tx);
             connection
                 .outgoing_tx
-                .unbounded_send(proto::Message::Envelope(request.into_envelope(
-                    message_id,
-                    None,
-                    original_sender_id.map(Into::into),
-                )))
+                .unbounded_send(proto::Message::Envelope(envelope))
                 .map_err(|_| anyhow!("connection was closed"))?;
             Ok(())
         });
@@ -491,19 +477,10 @@ impl Peer {
             send?;
             let (response, received_at, _barrier) =
                 rx.await.map_err(|_| anyhow!("connection was closed"))?;
-
             if let Some(proto::envelope::Payload::Error(error)) = &response.payload {
-                Err(RpcError::from_proto(&error, T::NAME))
-            } else {
-                Ok(TypedEnvelope {
-                    message_id: response.id,
-                    sender_id: receiver_id,
-                    original_sender_id: response.original_sender_id,
-                    payload: T::Response::from_envelope(response)
-                        .ok_or_else(|| anyhow!("received response of the wrong type"))?,
-                    received_at,
-                })
+                return Err(RpcError::from_proto(&error, type_name));
             }
+            Ok((response, received_at))
         }
     }
 
@@ -605,7 +582,7 @@ impl Peer {
         receipt: Receipt<T>,
         response: T::Response,
     ) -> Result<()> {
-        let connection = self.connection_state(receipt.sender_id)?;
+        let connection = self.connection_state(receipt.sender_id.into())?;
         let message_id = connection
             .next_message_id
             .fetch_add(1, atomic::Ordering::SeqCst);
@@ -620,7 +597,7 @@ impl Peer {
     }
 
     pub fn end_stream<T: RequestMessage>(&self, receipt: Receipt<T>) -> Result<()> {
-        let connection = self.connection_state(receipt.sender_id)?;
+        let connection = self.connection_state(receipt.sender_id.into())?;
         let message_id = connection
             .next_message_id
             .fetch_add(1, atomic::Ordering::SeqCst);
@@ -642,7 +619,7 @@ impl Peer {
         receipt: Receipt<T>,
         response: proto::Error,
     ) -> Result<()> {
-        let connection = self.connection_state(receipt.sender_id)?;
+        let connection = self.connection_state(receipt.sender_id.into())?;
         let message_id = connection
             .next_message_id
             .fetch_add(1, atomic::Ordering::SeqCst);
@@ -660,7 +637,7 @@ impl Peer {
         &self,
         envelope: Box<dyn AnyTypedEnvelope>,
     ) -> Result<()> {
-        let connection = self.connection_state(envelope.sender_id())?;
+        let connection = self.connection_state(envelope.sender_id().into())?;
         let response = ErrorCode::Internal
             .message(format!(
                 "message {} was not handled",
@@ -703,7 +680,6 @@ impl Serialize for Peer {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::TypedEnvelope;
     use async_tungstenite::tungstenite::Message as WebSocketMessage;
     use gpui::TestAppContext;
 
