@@ -84,7 +84,7 @@ pub fn init(themes_to_load: LoadThemes, cx: &mut AppContext) {
         let buffer_font_size = ThemeSettings::get_global(cx).buffer_font_size;
         if buffer_font_size != prev_buffer_font_size {
             prev_buffer_font_size = buffer_font_size;
-            reset_font_size(cx);
+            reset_buffer_font_size(cx);
         }
     })
     .detach();
