@@ -9,10 +9,12 @@ use gpui::WindowContext;
 use language::BracketPair;
 use serde::Deserialize;
 use std::sync::Arc;
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum SurroundsType {
     Motion(Motion),
     Object(Object),
+    Selection,
 }
 
 // This exists so that we can have Deserialize on Operators, but not on Motions.
@@ -29,6 +31,7 @@ pub fn add_surrounds(text: Arc<str>, target: SurroundsType, cx: &mut WindowConte
     Vim::update(cx, |vim, cx| {
         vim.stop_recording();
         let count = vim.take_count(cx);
+        let mode = vim.state().mode;
         vim.update_active_editor(cx, |_, editor, cx| {
             let text_layout_details = editor.text_layout_details(cx);
             editor.transact(cx, |editor, cx| {
@@ -40,6 +43,7 @@ pub fn add_surrounds(text: Arc<str>, target: SurroundsType, cx: &mut WindowConte
                         start: text.to_string(),
                         end: text.to_string(),
                         close: true,
+                        surround: true,
                         newline: false,
                     },
                 };
@@ -83,19 +87,25 @@ pub fn add_surrounds(text: Arc<str>, target: SurroundsType, cx: &mut WindowConte
                                 });
                             range
                         }
+                        SurroundsType::Selection => Some(selection.range()),
                     };
 
                     if let Some(range) = range {
                         let start = range.start.to_offset(&display_map, Bias::Right);
                         let end = range.end.to_offset(&display_map, Bias::Left);
-                        let start_cursor_str =
-                            format!("{}{}", pair.start, if surround { " " } else { "" });
-                        let close_cursor_str =
-                            format!("{}{}", if surround { " " } else { "" }, pair.end);
+                        let (start_cursor_str, end_cursor_str) = if mode == Mode::VisualLine {
+                            (format!("{}\n", pair.start), format!("{}\n", pair.end))
+                        } else {
+                            let maybe_space = if surround { " " } else { "" };
+                            (
+                                format!("{}{}", pair.start, maybe_space),
+                                format!("{}{}", maybe_space, pair.end),
+                            )
+                        };
                         let start_anchor = display_map.buffer_snapshot.anchor_before(start);
 
                         edits.push((start..start, start_cursor_str));
-                        edits.push((end..end, close_cursor_str));
+                        edits.push((end..end, end_cursor_str));
                         anchors.push(start_anchor..start_anchor);
                     } else {
                         let start_anchor = display_map
@@ -110,7 +120,11 @@ pub fn add_surrounds(text: Arc<str>, target: SurroundsType, cx: &mut WindowConte
                 });
                 editor.set_clip_at_line_ends(true, cx);
                 editor.change_selections(Some(Autoscroll::fit()), cx, |s| {
-                    s.select_anchor_ranges(anchors)
+                    if mode == Mode::VisualBlock {
+                        s.select_anchor_ranges(anchors.into_iter().take(1))
+                    } else {
+                        s.select_anchor_ranges(anchors)
+                    }
                 });
             });
         });
@@ -227,6 +241,7 @@ pub fn change_surrounds(text: Arc<str>, target: Object, cx: &mut WindowContext)
                             start: text.to_string(),
                             end: text.to_string(),
                             close: true,
+                            surround: true,
                             newline: false,
                         },
                     };
@@ -388,54 +403,63 @@ fn all_support_surround_pair() -> Vec<BracketPair> {
             start: "{".into(),
             end: "}".into(),
             close: true,
+            surround: true,
             newline: false,
         },
         BracketPair {
             start: "'".into(),
             end: "'".into(),
             close: true,
+            surround: true,
             newline: false,
         },
         BracketPair {
             start: "`".into(),
             end: "`".into(),
             close: true,
+            surround: true,
             newline: false,
         },
         BracketPair {
             start: "\"".into(),
             end: "\"".into(),
             close: true,
+            surround: true,
             newline: false,
         },
         BracketPair {
             start: "(".into(),
             end: ")".into(),
             close: true,
+            surround: true,
             newline: false,
         },
         BracketPair {
             start: "|".into(),
             end: "|".into(),
             close: true,
+            surround: true,
             newline: false,
         },
         BracketPair {
             start: "[".into(),
             end: "]".into(),
             close: true,
+            surround: true,
             newline: false,
         },
         BracketPair {
             start: "{".into(),
             end: "}".into(),
             close: true,
+            surround: true,
             newline: false,
         },
         BracketPair {
             start: "<".into(),
             end: ">".into(),
             close: true,
+            surround: true,
             newline: false,
         },
     ];
@@ -461,48 +485,56 @@ fn object_to_bracket_pair(object: Object) -> Option<BracketPair> {
             start: "'".to_string(),
             end: "'".to_string(),
             close: true,
+            surround: true,
             newline: false,
         }),
         Object::BackQuotes => Some(BracketPair {
             start: "`".to_string(),
             end: "`".to_string(),
             close: true,
+            surround: true,
             newline: false,
         }),
         Object::DoubleQuotes => Some(BracketPair {
             start: "\"".to_string(),
             end: "\"".to_string(),
             close: true,
+            surround: true,
             newline: false,
         }),
         Object::VerticalBars => Some(BracketPair {
             start: "|".to_string(),
             end: "|".to_string(),
             close: true,
+            surround: true,
             newline: false,
         }),
         Object::Parentheses => Some(BracketPair {
             start: "(".to_string(),
             end: ")".to_string(),
             close: true,
+            surround: true,
             newline: false,
         }),
         Object::SquareBrackets => Some(BracketPair {
             start: "[".to_string(),
             end: "]".to_string(),
             close: true,
+            surround: true,
             newline: false,
         }),
         Object::CurlyBrackets => Some(BracketPair {
             start: "{".to_string(),
             end: "}".to_string(),
             close: true,
+            surround: true,
             newline: false,
         }),
         Object::AngleBrackets => Some(BracketPair {
             start: "<".to_string(),
             end: ">".to_string(),
             close: true,
+            surround: true,
             newline: false,
         }),
         _ => None,
@@ -511,9 +543,14 @@ fn object_to_bracket_pair(object: Object) -> Option<BracketPair> {
 
 #[cfg(test)]
 mod test {
+    use gpui::KeyBinding;
     use indoc::indoc;
 
-    use crate::{state::Mode, test::VimTestContext};
+    use crate::{
+        state::{Mode, Operator},
+        test::VimTestContext,
+        PushOperator,
+    };
 
     #[gpui::test]
     async fn test_add_surrounds(cx: &mut gpui::TestAppContext) {
@@ -663,6 +700,123 @@ mod test {
         );
     }
 
+    #[gpui::test]
+    async fn test_add_surrounds_visual(cx: &mut gpui::TestAppContext) {
+        let mut cx = VimTestContext::new(cx, true).await;
+
+        cx.update(|cx| {
+            cx.bind_keys([KeyBinding::new(
+                "shift-s",
+                PushOperator(Operator::AddSurrounds { target: None }),
+                Some("vim_mode == visual"),
+            )])
+        });
+
+        // test add surrounds with arround
+        cx.set_state(
+            indoc! {"
+            The quˇick brown
+            fox jumps over
+            the lazy dog."},
+            Mode::Normal,
+        );
+        cx.simulate_keystrokes("v i w shift-s {");
+        cx.assert_state(
+            indoc! {"
+            The ˇ{ quick } brown
+            fox jumps over
+            the lazy dog."},
+            Mode::Normal,
+        );
+
+        // test add surrounds not with arround
+        cx.set_state(
+            indoc! {"
+            The quˇick brown
+            fox jumps over
+            the lazy dog."},
+            Mode::Normal,
+        );
+        cx.simulate_keystrokes("v i w shift-s }");
+        cx.assert_state(
+            indoc! {"
+            The ˇ{quick} brown
+            fox jumps over
+            the lazy dog."},
+            Mode::Normal,
+        );
+
+        // test add surrounds with motion
+        cx.set_state(
+            indoc! {"
+            The quˇick brown
+            fox jumps over
+            the lazy dog."},
+            Mode::Normal,
+        );
+        cx.simulate_keystrokes("v e shift-s }");
+        cx.assert_state(
+            indoc! {"
+            The quˇ{ick} brown
+            fox jumps over
+            the lazy dog."},
+            Mode::Normal,
+        );
+
+        // test add surrounds with multi cursor
+        cx.set_state(
+            indoc! {"
+            The quˇick brown
+            fox jumps over
+            the laˇzy dog."},
+            Mode::Normal,
+        );
+        cx.simulate_keystrokes("v i w shift-s '");
+        cx.assert_state(
+            indoc! {"
+            The ˇ'quick' brown
+            fox jumps over
+            the ˇ'lazy' dog."},
+            Mode::Normal,
+        );
+
+        // test add surrounds with visual block
+        cx.set_state(
+            indoc! {"
+            The quˇick brown
+            fox jumps over
+            the lazy dog."},
+            Mode::Normal,
+        );
+        cx.simulate_keystrokes("ctrl-v i w j j shift-s '");
+        cx.assert_state(
+            indoc! {"
+            The ˇ'quick' brown
+            fox 'jumps' over
+            the 'lazy 'dog."},
+            Mode::Normal,
+        );
+
+        // test add surrounds with visual line
+        cx.set_state(
+            indoc! {"
+            The quˇick brown
+            fox jumps over
+            the lazy dog."},
+            Mode::Normal,
+        );
+        cx.simulate_keystrokes("j shift-v shift-s '");
+        cx.assert_state(
+            indoc! {"
+            The quick brown
+            ˇ'
+            fox jumps over
+            '
+            the lazy dog."},
+            Mode::Normal,
+        );
+    }
+
     #[gpui::test]
     async fn test_delete_surrounds(cx: &mut gpui::TestAppContext) {
         let mut cx = VimTestContext::new(cx, true).await;
