@@ -55,7 +55,7 @@ impl OpenRequest {
     fn parse_file_path(&mut self, file: &str) {
         if let Some(decoded) = urlencoding::decode(file).log_err() {
             if let Some(path_buf) =
-                PathLikeWithPosition::parse_str(&decoded, |s| PathBuf::try_from(s)).log_err()
+                PathLikeWithPosition::parse_str(&decoded, |_, s| PathBuf::try_from(s)).log_err()
             {
                 self.open_paths.push(path_buf)
             }
@@ -247,7 +247,7 @@ pub async fn handle_cli_connection(
                         Err(error) => {
                             responses
                                 .send(CliResponse::Stderr {
-                                    message: format!("{}", error),
+                                    message: format!("{error}"),
                                 })
                                 .log_err();
                             responses.send(CliResponse::Exit { status: 1 }).log_err();
@@ -263,7 +263,7 @@ pub async fn handle_cli_connection(
                 {
                     responses
                         .send(CliResponse::Stderr {
-                            message: format!("{}", e),
+                            message: format!("{e}"),
                         })
                         .log_err();
                     responses.send(CliResponse::Exit { status: 1 }).log_err();
@@ -295,7 +295,7 @@ pub async fn handle_cli_connection(
                         .map(|path_with_position_string| {
                             PathLikeWithPosition::parse_str(
                                 &path_with_position_string,
-                                |path_str| {
+                                |_, path_str| {
                                     Ok::<_, std::convert::Infallible>(
                                         Path::new(path_str).to_path_buf(),
                                     )
@@ -342,10 +342,7 @@ pub async fn handle_cli_connection(
                                     Some(Err(err)) => {
                                         responses
                                             .send(CliResponse::Stderr {
-                                                message: format!(
-                                                    "error opening {:?}: {}",
-                                                    path, err
-                                                ),
+                                                message: format!("error opening {path:?}: {err}"),
                                             })
                                             .log_err();
                                         errored = true;
@@ -392,13 +389,14 @@ pub async fn handle_cli_connection(
                             errored = true;
                             responses
                                 .send(CliResponse::Stderr {
-                                    message: format!("error opening {:?}: {}", paths, error),
+                                    message: format!("error opening {paths:?}: {error}"),
                                 })
                                 .log_err();
                         }
                     }
                 } else if matches!(KEY_VALUE_STORE.read_kvp(FIRST_OPEN), Ok(None)) {
-                    cx.update(|cx| show_welcome_view(app_state, cx)).log_err();
+                    cx.update(|cx| show_welcome_view(app_state, cx).detach())
+                        .log_err();
                 } else {
                     cx.update(|cx| {
                         workspace::open_new(app_state, cx, |workspace, cx| {
