@@ -22,4 +22,20 @@ pub struct OpenZedUrl {
 
 impl_actions!(zed, [OpenBrowser, OpenZedUrl]);
 
-actions!(zed, [OpenSettings, Quit]);
+actions!(
+    zed,
+    [
+        OpenSettings,
+        Quit,
+        OpenKeymap,
+        About,
+        OpenLicenses,
+        OpenTelemetryLog,
+        DecreaseBufferFontSize,
+        IncreaseBufferFontSize,
+        ResetBufferFontSize,
+        DecreaseUiFontSize,
+        IncreaseUiFontSize,
+        ResetUiFontSize
+    ]
+);
