@@ -1,4 +1,12 @@
 import { danger, warn } from "danger";
+const { prHygiene } = require("danger-plugin-pr-hygiene");
+
+prHygiene({
+  rules: {
+    // Don't enable this rule just yet, as it can have false positives.
+    useImperativeMood: "off",
+  },
+});
 
 const RELEASE_NOTES_PATTERN = new RegExp("Release Notes:\\r?\\n\\s+-", "gm");
 
